/*
 * ******************************************************************************************
 *
 * File		: apb_pit_driver.c
 * Author	: GOWIN
 * Function	: Driver definition of APB PIT
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_pit_driver.h"
#include "apb_pit_dev.h"
#include "apb_pit_hal.h"


// Definitions ------------------------------------------------------------------------------

// Initializes APB PIT
int apb_pit_init(void)
{
	/* Disable APB PIT */
	DEV_APB_PIT->CHNEN = 0;

	/* Set APB PIT control mode */
	DEV_APB_PIT->CHANNEL[0].CTRL = (APB_PIT_CHNCTRL_TMR_32BIT | APB_PIT_CHNCTRL_CLK_PCLK);
	DEV_APB_PIT->CHANNEL[1].CTRL = (APB_PIT_CHNCTRL_TMR_32BIT | APB_PIT_CHNCTRL_CLK_PCLK);
	DEV_APB_PIT->CHANNEL[2].CTRL = (APB_PIT_CHNCTRL_TMR_32BIT | APB_PIT_CHNCTRL_CLK_PCLK);
	DEV_APB_PIT->CHANNEL[3].CTRL = (APB_PIT_CHNCTRL_TMR_32BIT | APB_PIT_CHNCTRL_CLK_PCLK);

	/* Clear and disable interrupt */
	DEV_APB_PIT->INTEN = 0;
	DEV_APB_PIT->INTST = -1;

	/* Set APB PIT priority to lowest */
	hal_enable_interrupt(IRQ_APB_PIT_SOURCE);

	return APB_PIT_DRIVER_OK;
}

// Read time
unsigned int apb_pit_read(unsigned int tmr)
{
	if (tmr >= APB_PIT_NUMBER_OF_CHANNELS)
	{
		return 0;
	}

	return (DEV_APB_PIT->CHANNEL[tmr].RELOAD - DEV_APB_PIT->CHANNEL[tmr].COUNTER);
}

// Control
int apb_pit_control(unsigned int mode, unsigned int tmr)
{
	if (tmr >= APB_PIT_NUMBER_OF_CHANNELS)
	{
		return APB_PIT_DRIVER_ERROR_UNSUPPORTED;
	}

	if (mode & APB_PIT_TIMER_START)
	{
		// Start timer
		DEV_APB_PIT->CHNEN |= (0x1 << (4 * (tmr)));
	}
	else if (mode & APB_PIT_TIMER_STOP)
	{
		// Stop timer
		DEV_APB_PIT->CHNEN &= ~(0x1 << (4 * (tmr)));
	}
	else if (mode & APB_PIT_TIMER_INTR_ENABLE)
	{
		// Enable interrupt
		DEV_APB_PIT->INTEN |= (0x1 << (4 * (tmr)));
	}
	else if (mode & APB_PIT_TIMER_INTR_DISABLE)
	{
		// Disable interrupt
		DEV_APB_PIT->INTEN &= ~(0x1 << (4 * (tmr)));
	}
	else if (mode & APB_PIT_TIMER_INTR_CLEAR)
	{
		// Clear interrupt
		DEV_APB_PIT->INTST = 0xF << (4 * (tmr));
	}

	return APB_PIT_DRIVER_OK;
}

// Set period
int apb_pit_set_period(unsigned int tmr, unsigned int period)
{
	if (tmr >= APB_PIT_NUMBER_OF_CHANNELS)
	{
		return APB_PIT_DRIVER_ERROR_UNSUPPORTED;
	}

	DEV_APB_PIT->CHANNEL[tmr].RELOAD = period;

	return APB_PIT_DRIVER_OK;
}

// Get status
unsigned int apb_pit_get_status(unsigned int tmr)
{
	return (DEV_APB_PIT->INTST & (0xF << (4 * (tmr))));
}

// Get (m)s tick
unsigned int apb_pit_get_tick(unsigned int mode, unsigned int sec)
{
	if (mode & APB_PIT_TIMER_SEC_TICK)
	{
		// Second
		return sec * SYSCLK;
	}
	else if (mode & APB_PIT_TIMER_MSEC_TICK)
	{
		// Microsecond
		return sec * (SYSCLK / 1000);
	}
	else
	{
		return 0;
	}
}
