/*
 * ******************************************************************************************
 *
 * File		: apb_pwm_driver.h
 * Author	: GOWIN
 * Function	: Driver definition of APB PWM
 *
 * ******************************************************************************************
 */

#ifndef __APB_PWM_DRIVER_H__
#define __APB_PWM_DRIVER_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_pit.h"


// Definitions ------------------------------------------------------------------------------

/****** APB PWM Control Codes *****/

#define APB_PWM_CONTROL_Msk                 (0xFFUL)

#define APB_PWM_ACTIVE_CONFIGURE            (0x01UL)   // Configure APB PWM ; arg1 = APB PWM channel number

/*----- APB PWM Control Codes: Configure Parameters: PARK Mode -----*/
#define APB_PWM_PARK_BITS_Pos               8
#define APB_PWM_PARK_BITS_Msk               (1UL << APB_PWM_PARK_BITS_Pos)
#define APB_PWM_PARK_LOW                    (0UL << APB_PWM_PARK_BITS_Pos)    // Park LOW (default)
#define APB_PWM_PARK_HIGH                   (1UL << APB_PWM_PARK_BITS_Pos)    // Park HIGH

/*----- APB PWM Control Codes: Configure Parameters: Clock Source -----*/
#define APB_PWM_CLKSRC_Pos                  9
#define APB_PWM_CLKSRC_Msk                  (1UL << APB_PWM_CLKSRC_Pos)
#define APB_PWM_CLKSRC_SYSTEM               (0UL << APB_PWM_CLKSRC_Pos)       // System clock (default)
#define APB_PWM_CLKSRC_EXTERNAL             (1UL << APB_PWM_CLKSRC_Pos)       // External clock


/***********************************************************************
\brief APB PWM Status
************************************************************************/
typedef struct _APB_PWM_STATUS
{
	unsigned int configured    : 8;               // APB PWM channel state: 1=Configured, 0=Un-configured
	unsigned int output        : 8;               // APB PWM channel output: 1=Enable, 0=Disable
} APB_PWM_STATUS;


/****** APB PWM Event *****/
#define APB_PWM_EVENT_ONESTEP_COMPLETE      (1UL << 0)  // One step complete


/***********************************************************************
  \fn          void APB_PWM_SignalEvent (unsigned int event)
  \brief       Signal APB PWM Events.
  \param[in]   event  APB_PWM_events notification mask
  \return      none
************************************************************************/
typedef void (*APB_PWM_SignalEvent_t) (unsigned int ch_num, unsigned int event);  // Pointer to APB_PWM_SignalEvent : Signal APB PWM Event.


/***********************************************************************
\brief APB PWM Device Driver Capabilities.
************************************************************************/
typedef struct _APB_PWM_CAPABILITIES
{
	unsigned int channels               : 4;  // Supports APB PWM channel numbers
	unsigned int event_onestep_complete : 1;  // Signal one step complete event: APB_PWM_EVENT_ONESTEP_COMPLETE
} APB_PWM_CAPABILITIES;


// Number of APB PWM channels
#define APB_PWM_NUMBER_OF_CHANNELS           (4)

#define APB_PWM_EXTCLK_FREQ                  (32768)

// APB PIT Channel Control register
#define APB_PIT_CH_CTRL_PWMPARK_LOW          (0U << 4)
#define APB_PIT_CH_CTRL_PWMPARK_HIGH         (1U << 4)
#define APB_PIT_CH_CTRL_EXTCLK               (0U << 3)
#define APB_PIT_CH_CTRL_APBCLK               (1U << 3)
#define APB_PIT_CH_CTRL_MODE_PWM             (4U << 0)

// APB PWM MAX duty length
#define APB_PWM_DUTY_MAX                     255

// APB PWM flags
#define APB_PWM_FLAG_INITIALIZED             (1U << 0)
#define APB_PWM_FLAG_POWERED                 (1U << 1)
#define APB_PWM_FLAG_CHANNEL_ACTIVED(CHN)    (1U << (8 + CHN))

/**
\brief General power states
*/
typedef enum _APB_PWM_POWER_STATE
{
	APB_PWM_POWER_OFF,                      // Power off: no operation possible
	APB_PWM_POWER_LOW,                      // Low power mode: retain state, detect and signal wake-up events
	APB_PWM_POWER_FULL                      // Power on: full operation at maximum performance
} APB_PWM_POWER_STATE;

// APB PWM Information (Run-Time)
typedef struct _APB_PWM_INFO
{
	APB_PWM_SignalEvent_t   		cb_event;      						// Event callback
	unsigned int                	flags;         						// APB PWM driver flags
	unsigned int                	clk[APB_PWM_NUMBER_OF_CHANNELS];
	unsigned int                	freq[APB_PWM_NUMBER_OF_CHANNELS];   // APB PWM channel frequency
	unsigned int                	duty[APB_PWM_NUMBER_OF_CHANNELS];   // APB PWM channel duty cycle (0 ~ 255)
} APB_PWM_INFO;

// APB PWM Resources definitions
typedef struct
{
	APB_PWM_CAPABILITIES		capabilities;  // Capabilities
	APB_PIT_RegDef				*reg;          // Pointer to APB PIT register
	APB_PWM_INFO                *info;         // Run-Time Information
} const APB_PWM_RESOURCES;


//********************************************************
// Functions
//********************************************************

/*****************************************************************************
  \fn          APB_PWM_CAPABILITIES apb_pwm_get_capabilities (void)
  \brief       Get driver capabilities
  \return      APB_PWM_CAPABILITIES
******************************************************************************/
APB_PWM_CAPABILITIES apb_pwm_get_capabilities(void);

/*****************************************************************************
  \fn          int apb_pwm_initialize (APB_PWM_SignalEvent_t cb_event)
  \brief       Initialize APB PWM Interface.
  \param[in]   cb_event  Pointer to APB_PWM_SignalEvent
  \return      execution_status
******************************************************************************/
int apb_pwm_initialize(APB_PWM_SignalEvent_t cb_event);

/*****************************************************************************
  \fn          int apb_pwm_uninitialize (void)
  \brief       Uninitialized APB PWM Interface.
  \return      execution_status
******************************************************************************/
int apb_pwm_uninitialize(void);

/*****************************************************************************
  \fn          int apb_pwm_power_control (APB_PWM_POWER_STATE state)
  \brief       Control APB PWM Interface Power.
  \param[in]   state  Power state
  \return      execution_status
******************************************************************************/
int apb_pwm_power_control(APB_PWM_POWER_STATE  state);

/*****************************************************************************
  \fn          int apb_pwm_control (unsigned int control, unsigned int arg)
  \brief       Control APB PWM Interface.
  \param[in]   control  Operation
  \param[in]   arg      Argument of operation (optional)
  \return      common   execution_status and driver specific apb_pwm_execution_status
******************************************************************************/
int apb_pwm_control(unsigned int control, unsigned int arg);

/*****************************************************************************
  \fn          int apb_pwm_set_freq (unsigned char ch_num, unsigned int freq)
  \brief       Set APB PWM Interface frequency
  \param[in]   ch_num  APB PWM channel number
  \param[in]   freq    Frequency
  \return      execution_status
******************************************************************************/
int apb_pwm_set_freq(unsigned char ch_num, unsigned int freq);

/*****************************************************************************
  \fn          int apb_pwm_output (unsigned char ch_num, unsigned char duty)
  \brief       Output APB PWM pulse
  \param[in]   ch_num  APB PWM channel number
  \param[in]   duty    Duty length (0 ~ 255)
  \return      execution_status
******************************************************************************/
int apb_pwm_output(unsigned char ch_num, unsigned char duty);

/*****************************************************************************
  \fn          APB_PWM_STATUS apb_pwm_get_status (void)
  \brief       Get APB PWM status.
  \return      APB PWM status APB_PWM_STATUS
******************************************************************************/
APB_PWM_STATUS apb_pwm_get_status (void);


#ifdef __cplusplus
}
#endif


#endif	/* __APB_PWM_DRIVER_H__ */
