/*
 * ******************************************************************************************
 *
 * File		: apb_pwm_driver.c
 * Author	: GOWIN
 * Function	: Driver definition of APB PWM
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_pwm_driver.h"
#include "apb_pit_dev.h"
#include "apb_pit_hal.h"


// Definitions ------------------------------------------------------------------------------

#define APB_PWM_APBCLK_FREQ                  SYSCLK

// APB PWM information
static APB_PWM_INFO apb_pwm_info = {0};

// APB PWM resources
static const APB_PWM_RESOURCES apb_pwm =
{
	{
	  APB_PWM_NUMBER_OF_CHANNELS,	/* APB PWM channel numbers */
	  0                        		/* Signal one step complete event */
	},
	DEV_APB_PIT,
	&apb_pwm_info
};

// Output
static void apb_pwmout(unsigned char ch_num, unsigned int freq, unsigned int duty)
{
	unsigned int period, high_cycle, low_cycle;

	period = apb_pwm.info->clk[ch_num] / freq;
	high_cycle = ((period * duty) / APB_PWM_DUTY_MAX) - 1;
	low_cycle = period - high_cycle - 1;

	// Disable corresponding APB PWM channel bit
	apb_pwm.reg->CHNEN &= ~(1 << ((ch_num << 2) + 3));

	apb_pwm.reg->CHANNEL[ch_num].RELOAD = (high_cycle << 16) | (low_cycle);

	// Enable corresponding APB PWM channel bit
	apb_pwm.reg->CHNEN |= (1 << ((ch_num << 2) + 3));
}

/*****************************************************************************
  \fn          APB_PWM_CAPABILITIES apb_pwm_get_capabilities (void)
  \brief       Get driver capabilities
  \return      APB_PWM_CAPABILITIES
******************************************************************************/
APB_PWM_CAPABILITIES apb_pwm_get_capabilities(void)
{
	return apb_pwm.capabilities;
}

/*****************************************************************************
  \fn          int apb_pwm_initialize (APB_PWM_SignalEvent_t cb_event)
  \brief       Initialize APB PWM Interface.
  \param[in]   cb_event  Pointer to APB_PWM_SignalEvent
  \return      execution_status
******************************************************************************/
int apb_pwm_initialize(APB_PWM_SignalEvent_t cb_event)
{
	unsigned int ch_num;

	if (apb_pwm.info->flags & APB_PWM_FLAG_INITIALIZED)
	{
		// Driver is already initialized
		return APB_PIT_DRIVER_OK;
	}

	apb_pwm.info->cb_event = cb_event;

	for (ch_num = 0U; ch_num < APB_PWM_NUMBER_OF_CHANNELS; ch_num++)
	{
		apb_pwm.info->clk[ch_num] = 0;
		apb_pwm.info->freq[ch_num] = 0;
		apb_pwm.info->duty[ch_num] = 0;
	}

	apb_pwm.info->flags = APB_PWM_FLAG_INITIALIZED;

	return APB_PIT_DRIVER_OK;
}

/*****************************************************************************
  \fn          int apb_pwm_uninitialize (void)
  \brief       Uninitialized APB PWM Interface.
  \return      execution_status
******************************************************************************/
int apb_pwm_uninitialize(void)
{
	// Reset APB PWM status flags
	apb_pwm.info->flags = 0U;

	apb_pwm.info->cb_event = NULL;

	return APB_PIT_DRIVER_OK;
}

/*****************************************************************************
  \fn          int apb_pwm_power_control (APB_PWM_POWER_STATE state)
  \brief       Control APB PWM Interface Power.
  \param[in]   state  Power state
  \return      execution_status
******************************************************************************/
int apb_pwm_power_control(APB_PWM_POWER_STATE  state)
{
	unsigned int ch_num, ch_mask;

	switch (state)
	{
	case APB_PWM_POWER_OFF:
		for (ch_num = 0U, ch_mask = 0U; ch_num < APB_PWM_NUMBER_OF_CHANNELS; ch_num++)
		{
			if (apb_pwm.info->duty[ch_num])
			{
				ch_mask |= (1 << ((ch_num << 2) + 3));

				apb_pwm.info->clk[ch_num] = 0;
				apb_pwm.info->freq[ch_num] = 0;
				apb_pwm.info->duty[ch_num] = 0;
			}
		}

		// Disable corresponding APB PWM channel bit
		apb_pwm.reg->CHNEN &= ~(ch_mask);
		apb_pwm.info->flags &= ~APB_PWM_FLAG_POWERED;

		break;

	case APB_PWM_POWER_LOW:
		return APB_PIT_DRIVER_ERROR_UNSUPPORTED;

	case APB_PWM_POWER_FULL:
		if ((apb_pwm.info->flags & APB_PWM_FLAG_INITIALIZED) == 0U)
		{
			return APB_PIT_DRIVER_ERROR;
		}
		if ((apb_pwm.info->flags & APB_PWM_FLAG_POWERED) != 0U)
		{
			return APB_PIT_DRIVER_OK;
		}

		apb_pwm.info->flags = APB_PWM_FLAG_POWERED | APB_PWM_FLAG_INITIALIZED;

		break;

	default:
		return APB_PIT_DRIVER_ERROR_UNSUPPORTED;
	}

	return APB_PIT_DRIVER_OK;
}

/*****************************************************************************
  \fn          int apb_pwm_control (unsigned int control, unsigned int arg)
  \brief       Control APB PWM Interface.
  \param[in]   control  Operation
  \param[in]   arg      Argument of operation (optional)
  \return      common   execution_status and driver specific apb_pwm_execution_status
******************************************************************************/
int apb_pwm_control(unsigned int control, unsigned int arg)
{
	unsigned int ctrl, clk;

	if ((apb_pwm.info->flags & APB_PWM_FLAG_POWERED) == 0U)
	{
		// APB PWM not powered
		return APB_PIT_DRIVER_ERROR;
	}

	switch (control & APB_PWM_CONTROL_Msk)
	{
	case APB_PWM_ACTIVE_CONFIGURE:
		if (arg >= apb_pwm.capabilities.channels)
		{
			// APB PWM channel is not existed
			return APB_PIT_DRIVER_ERROR;
		}

		ctrl = APB_PIT_CH_CTRL_MODE_PWM;

		// APB PWM park value
		if (control & APB_PWM_PARK_HIGH)
		{
			ctrl |= APB_PIT_CH_CTRL_PWMPARK_HIGH;
		}

		// APB PWM clock source
		if (control & APB_PWM_CLKSRC_EXTERNAL)
		{
			clk = APB_PWM_EXTCLK_FREQ;
		}
		else
		{
			ctrl |= APB_PIT_CH_CTRL_APBCLK;
			clk = APB_PWM_APBCLK_FREQ;
		}

		apb_pwm.reg->CHANNEL[arg].CTRL = ctrl;
		apb_pwm.info->clk[arg] = clk;

		apb_pwm.info->flags |= APB_PWM_FLAG_CHANNEL_ACTIVED(arg);

		return APB_PIT_DRIVER_OK;

	default:
		// Unsupported command
		return APB_PIT_DRIVER_ERROR_UNSUPPORTED;
	}
}

/*****************************************************************************
  \fn          int apb_pwm_set_freq (unsigned char ch_num, unsigned int freq)
  \brief       Set APB PWM Interface frequency
  \param[in]   ch_num  APB PWM channel number
  \param[in]   freq    Frequency
  \return      execution_status
******************************************************************************/
int apb_pwm_set_freq(unsigned char ch_num, unsigned int freq)
{
	if (ch_num >= apb_pwm.capabilities.channels)
	{
		// APB PWM channel is not existed
		return APB_PIT_DRIVER_ERROR;
	}

	if ((apb_pwm.info->flags & APB_PWM_FLAG_CHANNEL_ACTIVED(ch_num)) == 0U)
	{
		// APB PWM channel is not activated
		return APB_PIT_DRIVER_ERROR;
	}

	// APB PWM duty cycle is NOT zero, output is enabled
	if (apb_pwm.info->duty[ch_num])
	{
		apb_pwmout(ch_num, freq, apb_pwm.info->duty[ch_num]);
	}

	apb_pwm.info->freq[ch_num] = freq;

	return APB_PIT_DRIVER_OK;
}

/*****************************************************************************
  \fn          int apb_pwm_output (unsigned char ch_num, unsigned char duty)
  \brief       Output APB PWM pulse
  \param[in]   ch_num  APB PWM channel number
  \param[in]   duty    Duty length (0 ~ 255)
  \return      execution_status
******************************************************************************/
int apb_pwm_output(unsigned char ch_num, unsigned char duty)
{
	if (ch_num >= apb_pwm.capabilities.channels)
	{
		// APB PWM channel is not existed
		return APB_PIT_DRIVER_ERROR;
	}

	if ((apb_pwm.info->flags & APB_PWM_FLAG_CHANNEL_ACTIVED(ch_num)) == 0U)
	{
		// APB PWM channel is not activated
		return APB_PIT_DRIVER_ERROR;
	}

	// Overflow Max. duty value (0 ~ 255)
	if (duty > APB_PWM_DUTY_MAX)
	{
		return APB_PIT_DRIVER_ERROR;
	}

	apb_pwm.info->duty[ch_num] = duty;

	if (duty)
	{
		apb_pwmout(ch_num, apb_pwm.info->freq[ch_num], duty);
	}
	else
	{
		// Duty is zero.
		// Disable corresponding APB PWM channel bit
		apb_pwm.reg->CHNEN &= ~(1 << ((ch_num << 2) + 3));
	}

	return APB_PIT_DRIVER_OK;
}

/*****************************************************************************
  \fn          APB_PWM_STATUS apb_pwm_get_status (void)
  \brief       Get APB PWM status.
  \return      APB PWM status APB_PWM_STATUS
******************************************************************************/
APB_PWM_STATUS apb_pwm_get_status (void)
{
	APB_PWM_STATUS stat;
	unsigned int ch_num;

	stat.configured = (apb_pwm.info->flags >> 8) & 0xFF;

	for (ch_num = stat.output = 0U; ch_num < APB_PWM_NUMBER_OF_CHANNELS; ch_num++)
	{
		if (apb_pwm.info->duty[ch_num])
		{
			stat.output |= (1 << ch_num);
		}
	}

	return stat;
}
